/////////////////////////////////////////////////////////////////////////////
// MODULE: Dib.h
// LANGUAJE: ANSI C
// COMPILER: LCC-Win32
// OS: Windows 9x, NT
//
// AUTHOR: Daniel Guerrero Miralles (daguer@geocities.com)
// LEGAL INFORMATION: This is PUBLIC DOMAIN source code.
//
// VERSION: 1.0.0 (March 24th, 1999)
// ABSTRACT: Functions to work with Device Independent Bitmaps (DIBs).
//
// NOTES:
// Some functions are based on the WinCapture example from the Win32 SDK.
// These function only accept DIBs in Win 3.0 format. OS/2 DIBs are not
// supported.
//

#ifndef _DIB_H_
#define _DIB_H_

#include <windows.h>

#define HDIB HANDLE

#define DIB_HasPalette(h) (DIB_BitsPerPixel((HDIB)h) <= 8)

// Load a DIB from an open file
//
// Parameter: - hFile: handle of the file
// Return value: on success, the handle of the DIB, NULL otherwise
//
// Notes: if the function succeed, the file pointer is set over the next
// byte past the DIB. If the function failed, the pointer is unchanged.

HDIB DIB_Load(HANDLE hFile);



// Load a DIB from a resource
//
// Parameter: - pszId: the resource identifier
// Return value: the handle of the DIB on success, NULL otherwise.
// Notes: DIB resources are stored as BITMAP resources.

HDIB DIB_FromResource(HMODULE hModule, LPTSTR pszId);



// Make a new copy of a DIB
//
// Parameter: - hDib: handle of the DIB
// Return value: NULL in error, or the handle of the new DIB on success.

HDIB DIB_Duplicate(HDIB hDib);



// Delete the DIB from memory
//
// Parameter: - hDib: handle of the DIB

void DIB_Release(HDIB hDib);



// Get the DIB color depth in bits per pixel (1, 4, 8 or 24).
//
// Parameter: - hDib: handle of the DIB
// Return value: the size of the DIB pixel

int DIB_BitsPerPixel(HDIB hDib);



// Get the DIB width in pixels.
//
// Parameter: - hDib: handle of the DIB
// Return value: the DIB width in pixels

int DIB_Width(HDIB hDib);



// Get the DIB height in pixels.
//
// Parameter: - hDib: handle of the DIB
// Return value: the DIB height in pixels.

int DIB_Height(HDIB hDib);



// Make a GDI palette object from the DIB palette.
//
// Parameter: - hDib: handle of the DIB
// Return value: the handle of the GDI palette (NULL on error).

HPALETTE DIB_GetPalette (HDIB hDib);



// Paint a DIB into a device at specified coords.
//
// Parameters: - hDib: handle of the DIB
//             - hDC:  handle of the device contest to paint in
//             - x:    horizontal positon of the DIB
//             - y:    vertical position of the DIB
// Return value: TRUE on success, FALSE on error.

BOOL DIB_Paint(HDIB hDib, HDC hDC, int x, int y);



// Paint a DIB into a device at specified coords and size
//
// Parameters: - hDib:    handle of the DIB
//             - hDC:     handle of the device contest to paint in
//             - x:       horizontal positon of the DIB
//             - y:       vertical position of the DIB
//             - nWidth:  desired width of the DIB
//             - nHeight: desired height of the DIB
// Return value: TRUE on success, FALSE otherwise.

BOOL DIB_Stretch(HDIB hDib, HDC hDC, int x, int y, int nWidth, int nHeight);



// Generate a new DIB with specified width and height
//
// Parameters: - hDib:    handle of the source DIB
//             - nWidth:  desired width for the new DIB
//             - hHeight: desired height for the new DIB
//             - uFlags:  a conbination of DRF_ flags, see notes.
// Return value: the handle of the new DIB on success, NULL otherwise.
//
// Notes: if the DRF_KEEPASPECTRATIO flag is specified the nHeight parameter
// is ignored, and the DIB is enlarged or shrinked vertically in the same
// proportion as horizontally. If the DRF_FAST flag is set, the fastest
// resizing method (point sampling) will be chosen. By default the function
// uses the best-looking method (linear interpolation). The resulting DIB
// will have the same color depth as the source DIB, but it may have a
// different palette.

//HDIB DIB_Resize(HDIB hDib, int nWidth int nHeight, UINT uFlags);




#endif // _DIB_H_
