/////////////////////////////////////////////////////////////////////////////
// MODULE: Dib.c
// LANGUAJE: ANSI C
// COMPILER: LCC-Win32
// OS: Windows 9x, NT
//
// AUTHOR: Daniel Guerrero Miralles (daguer@geocitiescom)
// LEGAL INFORMATION: This is PUBLIC DOMAIN source code.
//
// VERSION: 1.0.0 (March 28th, 1999)
// ABSTRACT: Functions to work with Device Independent Bitmaps (DIBs).
//
// NOTES:
// See Dib.h for details on functions parameters.
// Functions beginning with "DIB_" expect a handle to DIB as paramter, while
// these beginning with "BI_" expect a pointer to the BITMAPINFO structure.
//

#include <windows.h>
#include "dib.h"

// Constants ////////////////////////////////////////////////////////////////

#define DIB_SIGNATURE 0x4D42 // "BM"
#define DIB_PALVERSION 0x0300

// Macros ///////////////////////////////////////////////////////////////////

// -- generic macros --

#ifndef ALIGN
#define ALIGN(n,m) (((m) + (n) - 1) & -(n))
#endif

// -- BITMAPINFO macros --

#define BI_Width(p)        (((PBITMAPINFO)(p))->bmiHeader.biWidth)
#define BI_Height(p)       (((PBITMAPINFO)(p))->bmiHeader.biHeight)
#define BI_BitsPerPixel(p) (((PBITMAPINFO)(p))->bmiHeader.biBitCount)
#define BI_ColorsUsed(p)   (((PBITMAPINFO)(p))->bmiHeader.biClrUsed)
#define BI_PalSize(p)      (BI_NumPalColors(p) * sizeof(RGBQUAD))
#define BI_PalColors(p)    (&((PBITMAPINFO)(p))->bmiColors[0])
#define BI_Bits(p)         (((PBYTE)(p)) + sizeof(BITMAPINFOHEADER) + BI_PalSize(p))
#define BI_ImageSize(p)    (((PBITMAPINFO)(p))->bmiHeader.biSizeImage)
#define BI_IsBottomUp(p)   (BI_Height(p) > 0)

// Helper functions /////////////////////////////////////////////////////////

// -- generic functions --

static BOOL GetFilePointerQ(HANDLE hFile, PLARGE_INTEGER pqPointer)
{
	pqPointer->HighPart = 0;
	pqPointer->LowPart = SetFilePointer(hFile, 0, &pqPointer->HighPart, FILE_BEGIN);

	if (pqPointer->LowPart == (DWORD) -1)
	{
		return (GetLastError() == NO_ERROR);
	}

	return TRUE;
}

static BOOL SetFilePointerQ(HANDLE hFile, LARGE_INTEGER qNewPtr, DWORD dwFrom)
{
	DWORD dwResult;

	dwResult = SetFilePointer(hFile, qNewPtr.LowPart, &qNewPtr.HighPart, dwFrom);
	if (dwResult == (DWORD) -1)
	{
		return GetLastError() == NO_ERROR;
	}

	return TRUE;
}

static BOOL ReadFileSync(HANDLE hFile, PVOID pBuffer, DWORD dwSize)
{
	DWORD dwBytesRead;

	if (!ReadFile(hFile, pBuffer, dwSize, &dwBytesRead, NULL))
	{
		return FALSE;
	}
	if (dwBytesRead != dwSize)
	{
		return FALSE;
	}

	return TRUE;
}

// -- BITMAPINFO functions --

static int BI_NumPalColors(PBITMAPINFO pBI)
{
	if (BI_ColorsUsed(pBI) != 0)
	{
		return BI_ColorsUsed(pBI);
	}

	if (BI_BitsPerPixel(pBI) <= 8)
	{
		return (1 << BI_BitsPerPixel(pBI));
	}

	return 0;
}

static HPALETTE BI_GetPalette(PBITMAPINFO pBI)
{
	PLOGPALETTE pLogPal;
	HPALETTE hPal;
	int nNumColors, i;
	RGBQUAD * argbColor;

	nNumColors = BI_NumPalColors(pBI);
	if (nNumColors == 0)
	{
		return (HPALETTE) NULL;
	}

	pLogPal = (PLOGPALETTE) GlobalAlloc(GPTR, sizeof(LOGPALETTE)
		+ (sizeof(PALETTEENTRY) * nNumColors));

	if (pLogPal == NULL)
	{
		return (HPALETTE) NULL;
	}

	pLogPal->palVersion = DIB_PALVERSION;
	pLogPal->palNumEntries = nNumColors;

	argbColor = BI_PalColors(pBI);
	for (i = 0; i < nNumColors; i++)
	{
		pLogPal->palPalEntry[i].peRed = argbColor[i].rgbRed;
		pLogPal->palPalEntry[i].peGreen = argbColor[i].rgbGreen;
		pLogPal->palPalEntry[i].peBlue = argbColor[i].rgbBlue;
		pLogPal->palPalEntry[i].peFlags = 0; //PC_NOCOLLAPSE;
	}

	hPal = CreatePalette(pLogPal);

	GlobalFree(pLogPal);

	return hPal;
}

static int BI_BitsSize(PBITMAPINFO pBI)
{
	if (BI_ImageSize(pBI) != 0)
	{
		return BI_ImageSize(pBI);
	}

	return ((BI_Width(pBI) * BI_Height(pBI) * BI_BitsPerPixel(pBI)) / 8);
}

// Interface functions //////////////////////////////////////////////////////

int DIB_BitsPerPixel(HDIB hDib)
{
	PBITMAPINFO pBI;
	int nBPP;

	if (hDib == NULL)
		return 0;

	pBI = (PBITMAPINFO) GlobalLock(hDib);
	nBPP = BI_BitsPerPixel(pBI);
	GlobalUnlock(hDib);

	return nBPP;
}

int DIB_Width(HDIB hDib)
{
	PBITMAPINFO pBI;
	int nWidth;

	if (hDib == NULL)
		return 0;

	pBI = (PBITMAPINFO) GlobalLock(hDib);
	nWidth = BI_Width(pBI);
	GlobalUnlock(hDib);

	return nWidth;
}

int DIB_Height(HDIB hDib)
{
	PBITMAPINFO pBI;
	int nHeight;

	if (hDib == NULL)
		return 0;

	pBI = (PBITMAPINFO) GlobalLock(hDib);
	nHeight = BI_Height(pBI);
	GlobalUnlock(hDib);

	if (nHeight < 0)
	{
		return -nHeight;
	}

	return nHeight;
}

HPALETTE DIB_GetPalette (HDIB hDib)
{
	PBITMAPINFO pBI;
	HPALETTE hPal;

	if (hDib == NULL)
		return (HPALETTE) NULL;

	pBI = (PBITMAPINFO) GlobalLock(hDib);
	hPal = BI_GetPalette(pBI);
	GlobalUnlock(hDib);

	return hPal;
}

HDIB DIB_Load(HANDLE hFile)
{
	BITMAPFILEHEADER bfh;
	LARGE_INTEGER qStartPos, qTmp; // 64-bit integer (quad) to store file offset.
	HGLOBAL hgTmp;
	PBITMAPINFO pBI;
	BOOL fDibIsLocked;
	HDIB hDib = (HDIB) NULL;

	if (!GetFilePointerQ(hFile, &qStartPos))
		goto Error_Exit;

	if (!ReadFileSync(hFile, &bfh, sizeof(BITMAPFILEHEADER)))
		goto Error_CleanUp;

	if (bfh.bfType != DIB_SIGNATURE)
		goto Error_CleanUp;

	hDib = (HDIB) GlobalAlloc(GMEM_MOVEABLE, sizeof(BITMAPINFO));
	if (hDib == NULL)
		goto Error_CleanUp;

	pBI = (PBITMAPINFO) GlobalLock(hDib);
	fDibIsLocked = TRUE;
	if (!ReadFileSync(hFile, pBI, sizeof(BITMAPINFOHEADER)))
		goto Error_CleanUp;

	if (pBI->bmiHeader.biSize != sizeof(BITMAPINFOHEADER))
		goto Error_CleanUp;

	GlobalUnlock((HGLOBAL) hDib);
	fDibIsLocked = FALSE;
	hgTmp = GlobalReAlloc((HGLOBAL) hDib, sizeof(BITMAPINFOHEADER)
		+ BI_PalSize(pBI) + BI_BitsSize(pBI), 0);
	if (hgTmp == NULL)
		goto Error_CleanUp;

	hDib = (HDIB) hgTmp;
	pBI = (PBITMAPINFO) GlobalLock ((HGLOBAL) hDib);
	fDibIsLocked = TRUE;

	if (BI_PalSize(pBI) > 0)
	{

		if (!ReadFileSync(hFile, BI_PalColors(pBI), BI_PalSize(pBI)))
			goto Error_CleanUp;
	}

	qTmp.HighPart = 0;
	qTmp.LowPart = bfh.bfOffBits - BI_PalSize(pBI) - sizeof(BITMAPINFOHEADER);
	if (!SetFilePointerQ(hFile, qTmp, FILE_CURRENT))
		goto Error_CleanUp;

	if (ReadFileSync(hFile, BI_Bits(pBI), BI_BitsSize(pBI)))
	{
		GlobalUnlock((HGLOBAL) hDib);
		return hDib;
	}

Error_CleanUp:
	if (hDib != (HDIB) NULL)
	{
		if (fDibIsLocked)
			GlobalUnlock((HGLOBAL) hDib);

		GlobalFree((HGLOBAL) hDib);
	}
	SetFilePointerQ(hFile, qStartPos, FILE_BEGIN);

Error_Exit:
	return NULL;

}

HDIB DIB_FromResource(HMODULE hModule, LPTSTR pszId)
{
	HRSRC hRes;
	HGLOBAL hLoadedRes;
	PBYTE pbLockedRes;
	PBYTE pbLockedDib;
	DWORD dwResSize;
	HDIB hDib;

	hRes = FindResource(hModule, pszId, RT_BITMAP);
	if (hRes == NULL)
		return NULL;

	dwResSize = SizeofResource(hModule, hRes);
	hDib = GlobalAlloc(GMEM_MOVEABLE, dwResSize);
	if (hDib == NULL)
		return NULL;

	pbLockedDib = GlobalLock(hDib);

	hLoadedRes = (HDIB) LoadResource(hModule, hRes);
	pbLockedRes = LockResource(hLoadedRes);
	CopyMemory(pbLockedDib, pbLockedRes, dwResSize);

	GlobalUnlock(hDib);
	return hDib;
}

HDIB DIB_Duplicate(HDIB hDib)
{
	DWORD dwSize;
	HDIB hNewDib;
	PBYTE pbSrc;
	PBYTE pbDest;

	if (hDib == (HDIB)NULL)
	{
		return (HDIB)NULL;
	}

	dwSize = GlobalSize((HGLOBAL)hDib);
	hNewDib = (HDIB)GlobalAlloc(GMEM_MOVEABLE, dwSize);
	if (hNewDib == (HDIB)NULL)
	{
		return (HDIB)NULL;
	}

	pbSrc = GlobalLock((HGLOBAL)hDib);
	pbDest = GlobalLock((HGLOBAL)hNewDib);
	CopyMemory(pbDest, pbSrc, dwSize);
	GlobalUnlock(hDib);
	GlobalUnlock(hNewDib);

	return hNewDib;
}

void DIB_Release(HDIB hDib)
{
	GlobalFree((HGLOBAL) hDib);
}

BOOL DIB_Paint(HDIB hDib, HDC hDC, int x, int y)
{
	PBITMAPINFO pBI;
	int nXSrc, nYSrc;
	UINT uStartScan, uNumScans;
	int nResult;

	if (hDib == NULL)
		return FALSE;

	nXSrc = 0;
	uStartScan = 0;

	pBI = (PBITMAPINFO) GlobalLock((HGLOBAL) hDib);
	if (BI_IsBottomUp(pBI))
	{
		nYSrc = 0;
		uNumScans = BI_Height(pBI);
	}
	else
	{
		nYSrc = -BI_Height(pBI);
		uNumScans = -BI_Height(pBI);
	}

	nResult = SetDIBitsToDevice(hDC, x, y, BI_Width(pBI), BI_Height(pBI)
		, nXSrc, nYSrc, uStartScan, uNumScans, BI_Bits(pBI), pBI
		, DIB_RGB_COLORS);

	GlobalUnlock ((HGLOBAL) hDib);
	return (nResult != GDI_ERROR);
}

BOOL DIB_Stretch(HDIB hDib, HDC hDC, int x, int y, int nWidth, int nHeight)
{
	PBITMAPINFO pBI;
	int nSrcHeight;
	int nResult;

	if (hDib == NULL)
		return FALSE;

	SetStretchBltMode(hDC, COLORONCOLOR);

	pBI = (PBITMAPINFO) GlobalLock((HGLOBAL) hDib);
	if (BI_IsBottomUp(pBI))
	{
		nSrcHeight = BI_Height(pBI);
	}
	else
	{
		nSrcHeight = -BI_Height(pBI);
	}

	nResult = StretchDIBits(hDC, x, y, nWidth, nHeight, 0, 0, BI_Width(pBI)
		, nSrcHeight, BI_Bits(pBI), pBI, DIB_RGB_COLORS, SRCCOPY);

	GlobalUnlock ((HGLOBAL) hDib);
	return (nResult != GDI_ERROR);
}
